import { BrowserWindow, ipcMain, shell, BrowserWindowConstructorOptions } from 'electron'
import Constants from './utils/Constants'

/*
 * IPC Communications
 * */
export default class IPCs {
  static browserWindows: Map<string, BrowserWindow[]> = new Map()

  static initialize(window: BrowserWindow): void {
    ipcMain.on('mesGetFilePath', () => {
      if (Constants.IS_DEV_ENV) {
        window.webContents.send('msgReceivedFilePath', `${Constants.APP_INDEX_URL_DEV}`)
      } else {
        window.webContents.send('msgReceivedFilePath', Constants.APP_INDEX_URL_PROD)
      }
    })

    // Get application version
    ipcMain.on('msgRequestGetVersion', () => {
      window.webContents.send('msgReceivedVersion', Constants.APP_VERSION)
    })

    // Open url via web browser
    ipcMain.on('msgOpenExternalLink', async (event, url: string) => {
      await shell.openExternal(url)
    })

    // open new window
    ipcMain.on(
      'openWindow',
      async (
        event,
        url: string,
        options: BrowserWindowConstructorOptions & { isCloseOther: boolean }
      ) => {
        const ops = Object.assign(
          {},
          {
            isCloseOther: true,
            frame: false,
            useContentSize: true,
            webPreferences: Constants.DEFAULT_WEB_PREFERENCES
          },
          options
        )

        if (IPCs.browserWindows.has(url) && ops.isCloseOther) {
          const wins = IPCs.browserWindows.get(url)
          wins?.forEach((w) => !w.isDestroyed() && w.close())
          IPCs.browserWindows.set(url, [])
        }

        const win = new BrowserWindow(ops)

        win.setMenu(null)

        win.once('ready-to-show', (): void => {
          win.setAlwaysOnTop(true)
          win.show()
          win.focus()
          win.setAlwaysOnTop(false)
        })

        win.webContents.on('did-frame-finish-load', (): void => {
          if (Constants.IS_DEV_ENV) {
            win.webContents.openDevTools()
          }
        })

        await win.loadURL(url)

        if (!IPCs.browserWindows.has(url)) {
          IPCs.browserWindows.set(url, [])
        }

        IPCs.browserWindows.get(url)?.push(win)

        return win
      }
    )

    ipcMain.on('openDevTools', async (event, isOpen: boolean) => {
      if (isOpen) {
        window.webContents.openDevTools()
        IPCs.browserWindows.forEach((wins) => {
          wins.forEach((w) => {
            !w.isDestroyed() && w.webContents.openDevTools()
          })
        })
      } else {
        window.webContents.closeDevTools()
        IPCs.browserWindows.forEach((wins) => {
          wins.forEach((w) => {
            !w.isDestroyed() && w.webContents.closeDevTools()
          })
        })
      }
    })
  }
}
